SystemOrganization addCategory: #Gofer!

Object subclass: #Gofer
	instanceVariableNames: 'actions repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!Gofer commentStamp: 'lr 7/8/2009 11:53' prior: 0!
Gofer loads and unloads Monticello packages.

When loading a package with Gofer, it ...
- cleanly loads one or more packages from one or more repositories,
- finds the "latest" version of a package matching the given name prefix,
- makes sure that dependencies are correctly loaded as well,
- makes sure that the repository is assigned to the working copy, and
- makes sure that the repository is shared among all packages of the same repository. 

When unloading a package with Gofer, it ...
- cleanly unloads one or more packages from the image,
- detects dependencies and automatically unloads these as well,
- removes class categories and method protocols that belonged to the packages,
- unregisters repositories that are no longer in use.

" Example: Load Seaside 2.8 "
Gofer new
	url: 'http://www.squeaksource.com/KomHttpServer';
		load: 'DynamicBindings';
		load: 'KomServices';
		load: 'KomHttpServer';
	url: 'http://www.squeaksource.com/Seaside';
		load: 'Seaside2.8a';
		load: 'Scriptaculous';
	execute.
	
" Example: Unload Seaside 2.8 "
Gofer new
	unload: 'DynamicBindings';
	unload: 'KomServices';
	unload: 'KomHttpServer';
	unload: 'Seaside2';
	unload: 'Scriptaculous';
	execute.!

!Gofer methodsFor: 'accessing' stamp: 'lr 7/5/2009 10:13'!
actions
	^ actions! !

!Gofer methodsFor: 'private' stamp: 'lr 7/10/2009 17:13'!
basicCleanup
	MCFileBasedRepository flushAllCaches.
	MCWorkingCopy changed: #allManagers.
	self initialize! !

!Gofer methodsFor: 'private' stamp: 'lr 7/5/2009 09:55'!
basicExecute
	| loader |
	loader := MCMultiPackageLoader new.
	actions do: [ :action | action preExecute ].
	actions do: [ :action | action execute: loader ].
	loader load.
	actions do: [ :action | action postExecute ]! !

!Gofer methodsFor: 'options-repositories' stamp: 'lr 7/10/2009 16:27'!
croquet: aString
	self url: 'http://hedgehog.software.umn.edu:8888/' , aString! !

!Gofer methodsFor: 'actions' stamp: 'tg 7/29/2009 15:10'!
custom: aBlock
	"Execute the script from aBlock"
	
	actions addLast: (GoferCustom with: aBlock) ! !

!Gofer methodsFor: 'actions' stamp: 'tg 7/28/2009 12:07'!
execute
	"Perform the selected load and unload actions."

	actions isEmpty ifFalse: [ 
		self 
			basicExecute; 
			basicCleanup ]! !

!Gofer methodsFor: 'options-repositories' stamp: 'lr 7/10/2009 16:27'!
impara: aString
	self url: 'http://source.impara.de/' , aString! !

!Gofer methodsFor: 'initialization' stamp: 'lr 7/4/2009 16:59'!
initialize
	actions := OrderedCollection new! !

!Gofer methodsFor: 'actions' stamp: 'lr 7/4/2009 20:31'!
load: aString
	"Install the package named aString in the image."

	actions addLast: (GoferLoad on: self name: aString)! !

!Gofer methodsFor: 'options-repositories' stamp: 'lr 7/10/2009 16:25'!
renggli: aString
	self url: 'http://source.lukas-renggli.ch/' , aString! !

!Gofer methodsFor: 'accessing' stamp: 'lr 7/4/2009 18:06'!
repository
	^ repository! !

!Gofer methodsFor: 'options' stamp: 'lr 7/5/2009 10:20'!
repository: aRepository
	"Set the reposiory aRepository to load from during the following load actions."

	MCRepositoryGroup default
		addRepository: aRepository.
	repository := MCRepositoryGroup default repositories
		detect: [ :each | each = aRepository ]
		ifNone: [ nil ]! !

!Gofer methodsFor: 'options-repositories' stamp: 'lr 7/10/2009 16:29'!
saltypickle: aString
	self url: 'http://squeak.saltypickle.com/' , aString! !

!Gofer methodsFor: 'options-repositories' stamp: 'lr 7/10/2009 16:28'!
squeakfoundation: aString
	self url: 'http://source.squeakfoundation.org/' , aString! !

!Gofer methodsFor: 'options-repositories' stamp: 'lr 7/10/2009 16:28'!
squeaksource: aString
	self url: 'http://www.squeaksource.com/' , aString! !

!Gofer methodsFor: 'actions' stamp: 'lr 7/4/2009 20:31'!
unload: aString
	"Remove the package named aString from the image."

	actions addLast: (GoferUnload on: self name: aString)! !

!Gofer methodsFor: 'options' stamp: 'lr 7/5/2009 09:58'!
url: anUrlString
	"Convenience method to set a repository URL using anUrlString."

	self repository: (MCHttpRepository
		location: anUrlString
		user: String new
		password: String new)! !

!Gofer methodsFor: 'options-repositories' stamp: 'lr 7/10/2009 16:26'!
wiresong: aString
	self url: 'http://source.wiresong.ca/' , aString! !

Object subclass: #GoferAction
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferAction class methodsFor: 'instance creation' stamp: 'lr 7/4/2009 17:02'!
on: aGopher name: aString
	^ self basicNew initializeOn: aGopher name: aString! !

!GoferAction methodsFor: 'actions' stamp: 'lr 7/4/2009 17:09'!
execute: aLoader
	self subclassResponsibility! !

!GoferAction methodsFor: 'initialization' stamp: 'lr 7/4/2009 17:29'!
initializeOn: aGopher name: aString
	name := aString! !

!GoferAction methodsFor: 'accessing' stamp: 'lr 7/4/2009 18:13'!
name
	^ name! !

!GoferAction methodsFor: 'actions' stamp: 'lr 7/4/2009 17:18'!
postExecute! !

!GoferAction methodsFor: 'actions' stamp: 'lr 7/4/2009 17:29'!
preExecute! !

GoferAction subclass: #GoferCustom
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferCustom class methodsFor: 'class initialization' stamp: 'tg 7/29/2009 15:10'!
with: aBlock
	^ self new action: aBlock; yourself! !

!GoferCustom methodsFor: 'accessing' stamp: 'tg 7/29/2009 15:05'!
action
	^ action! !

!GoferCustom methodsFor: 'accessing' stamp: 'tg 7/29/2009 15:05'!
action: anObject
	action := anObject! !

!GoferCustom methodsFor: 'accessing' stamp: 'tg 7/29/2009 15:06'!
execute: aLoader
	self action notNil ifTrue: [ self action value ]! !

GoferAction subclass: #GoferLoad
	instanceVariableNames: 'repository version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferLoad class methodsFor: 'instance creation' stamp: 'lr 7/5/2009 10:14'!
on: aGopher version: aVersion
	^ self new initializeOn: aGopher version: aVersion! !

!GoferLoad methodsFor: 'private' stamp: 'lr 7/5/2009 10:15'!
addDependenciesTo: aGofer
	self version allDependenciesDo: [ :each |
		aGofer actions addLast: (self class
			on: aGofer version: each) ]! !

!GoferLoad methodsFor: 'actions' stamp: 'lr 7/5/2009 10:25'!
execute: aLoader
	aLoader updatePackage: self package withSnapshot: self snapshot! !

!GoferLoad methodsFor: 'private' stamp: 'lr 7/5/2009 10:21'!
findLatestVersionIn: aRepository
	| versions |
	versions := aRepository allVersionNames
		select: [ :each | each beginsWith: self name ].
	versions := versions asSortedCollection: [ :a :b |
		(a copyAfterLast: $.) asNumber <= (b copyAfterLast: $.) asNumber ].
	versions isEmpty
		ifTrue: [ self error: 'No version named ' , self name printString , ' found' ].
	^ aRepository loadVersionFromFileNamed: versions last , '.mcz'! !

!GoferLoad methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:22'!
initializeOn: aGopher name: aString
	super initializeOn: aGopher name: aString.
	self 
		initializeOn: aGopher 
		version: (self findLatestVersionIn: aGopher repository).
	self addDependenciesTo: aGopher! !

!GoferLoad methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:13'!
initializeOn: aGopher version: aVersion
	repository := aGopher repository.
	version := aVersion! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/5/2009 10:25'!
package
	^ self workingCopy package! !

!GoferLoad methodsFor: 'actions' stamp: 'lr 7/4/2009 20:52'!
postExecute
	self workingCopy repositoryGroup 
		addRepository: self repository.
	self workingCopy loaded: self version! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/4/2009 18:32'!
repository
	^ repository! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/5/2009 10:25'!
snapshot
	^ self version snapshot! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/4/2009 18:24'!
version
	^ version! !

!GoferLoad methodsFor: 'accessing' stamp: 'lr 7/4/2009 20:49'!
workingCopy
	^ self version workingCopy! !

GoferAction subclass: #GoferUnload
	instanceVariableNames: 'workingCopy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferUnload class methodsFor: 'instance creation' stamp: 'lr 7/5/2009 10:40'!
on: aGopher workingCopy: aWorkingCopy
	^ self new initializeOn: aGopher workingCopy: aWorkingCopy! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/5/2009 10:39'!
addDependenciesTo: aGofer
	| index dependencies |
	index := 1.
	dependencies := OrderedCollection new.
	dependencies addAll: (self workingCopy requiredPackages
		collect: [ :each | each workingCopy ]).
	[ index between: 1 and: dependencies size ] whileTrue: [
		(dependencies at: index) requiredPackages do: [ :each |
			(dependencies includes: each workingCopy)
				ifFalse: [ dependencies addLast: each workingCopy ] ].
		index := index + 1 ].
	dependencies do: [ :each |
		aGofer actions addLast: (self class
			on: aGofer workingCopy: each) ]! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 17:44'!
cleanupCategories
	"Cleanup class categories, leftover from the unloading process."

	self packageInfo systemCategories do: [ :category |
		(SystemOrganization classesInCategory: category) isEmpty
			ifTrue: [ SystemOrganization removeSystemCategory: category ] ]! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 17:44'!
cleanupProtocols
	"Cleanup the method protocols, left over from method extensions."

	self packageInfo foreignClasses do: [ :class |
		(self packageInfo foreignExtensionCategoriesForClass: class) do: [ :category |
			(class organization listAtCategoryNamed: category) isEmpty
				ifTrue: [ class organization removeCategory: category ] ] ]! !

!GoferUnload methodsFor: 'actions' stamp: 'lr 7/4/2009 17:14'!
execute: aLoader
	aLoader unloadPackage: self package! !

!GoferUnload methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:29'!
initializeOn: aGopher name: aString
	super initializeOn: aGopher name: aString.
	self initializeOn: aGopher workingCopy: (MCWorkingCopy registry
		detect: [ :each | each packageName = aString ]
		ifNone: [ self error: 'Working copy ' , aString printString , ' not found' ]).
	self addDependenciesTo: aGopher! !

!GoferUnload methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:29'!
initializeOn: aGopher workingCopy: aWorkingCopy
	workingCopy := aWorkingCopy! !

!GoferUnload methodsFor: 'accessing' stamp: 'lr 7/4/2009 17:36'!
package
	^ self workingCopy package! !

!GoferUnload methodsFor: 'accessing' stamp: 'lr 7/4/2009 17:36'!
packageInfo
	^ self workingCopy packageInfo! !

!GoferUnload methodsFor: 'actions' stamp: 'lr 7/4/2009 18:04'!
postExecute
	self cleanupCategories.
	self cleanupProtocols.
	self unregisterWorkingCopy.
	self unregisterPackageInfo.
	self unregisterRepositories! !

!GoferUnload methodsFor: 'actions' stamp: 'lr 7/4/2009 17:27'!
preExecute
	self packageInfo classes do: [ :class |
		(class selectors includes: #unload)
			ifTrue: [ class unload ] ]! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 17:38'!
unregisterPackageInfo
	"Unregister the package information from the system."

	PackageOrganizer default
		unregisterPackage: self packageInfo! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 19:02'!
unregisterRepositories
	"Remove the repositories if no longer in use by any of the packages."

	self workingCopy repositoryGroup repositories allButFirst do: [ :repository |
		MCWorkingCopy allManagers do: [ :copy |
			(copy repositoryGroup includes: repository)
				ifTrue: [ ^ self ] ].
		MCRepositoryGroup default
			removeRepository: repository ]! !

!GoferUnload methodsFor: 'private' stamp: 'lr 7/4/2009 17:52'!
unregisterWorkingCopy
	"Unregister the working copy."

	MCWorkingCopy registry
		removeKey: self package
		ifAbsent: [ ] ! !

!GoferUnload methodsFor: 'accessing' stamp: 'lr 7/4/2009 17:36'!
workingCopy
	^ workingCopy! !

TestCase subclass: #GoferTest
	instanceVariableNames: 'gofer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gofer'!

!GoferTest methodsFor: 'running' stamp: 'lr 7/10/2009 16:32'!
setUp
	super setUp.
	gofer := Gofer new! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:51'!
testCroquet
	gofer croquet: 'Hermes'.
	self assert: gofer repository locationWithTrailingSlash = 'http://hedgehog.software.umn.edu:8888/Hermes/'! !

!GoferTest methodsFor: 'testing' stamp: 'tg 7/29/2009 15:17'!
testCustom
	| executed |
	gofer renggli: 'pieraddons'; load: 'Pier-Setup'.
	executed := false.
	gofer custom: [ executed := true ].
	self assert: gofer actions size = 2.
	self assert: gofer actions last class = GoferCustom.
	gofer actions last execute: nil.
	self assert: executed.! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:56'!
testImpara
	gofer impara: 'Tweak'.
	self assert: gofer repository locationWithTrailingSlash = 'http://source.impara.de/Tweak/'! !

!GoferTest methodsFor: 'testing' stamp: 'lr 7/10/2009 16:58'!
testInitialized
	self assert: gofer actions isEmpty! !

!GoferTest methodsFor: 'testing' stamp: 'lr 7/10/2009 17:16'!
testLoadUnload
	gofer renggli: 'pieraddons'; load: 'Pier-Setup'.
	self assert: gofer actions size = 1.
	self assert: gofer actions first repository = gofer repository.
	self assert: gofer actions first version notNil.
	self assert: gofer actions first package notNil.
	self assert: gofer actions first snapshot notNil.
	self assert: gofer actions first workingCopy notNil.
	self shouldnt: [ gofer execute ] raise: Error.
	
	gofer unload: 'Pier-Setup'.
	self assert: gofer actions size = 1.
	self assert: gofer actions first package notNil.
	self assert: gofer actions first packageInfo notNil.
	self assert: gofer actions first workingCopy notNil.
	self shouldnt: [ gofer execute ] raise: Error! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:56'!
testRenggli
	gofer renggli: 'pier'.
	self assert: gofer repository locationWithTrailingSlash = 'http://source.lukas-renggli.ch/pier/'! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:47'!
testRepository
	gofer repository: MCDirectoryRepository new.
	self assert: (gofer repository isKindOf: MCDirectoryRepository)! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:53'!
testSaltypickle
	gofer saltypickle: 'GraphViz'.
	self assert: gofer repository locationWithTrailingSlash = 'http://squeak.saltypickle.com/GraphViz/'! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:57'!
testSqueakfoundation
	gofer squeakfoundation: '39a'.
	self assert: gofer repository locationWithTrailingSlash = 'http://source.squeakfoundation.org/39a/'! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:54'!
testSqueaksource
	gofer squeaksource: 'Seaside29'.
	self assert: gofer repository locationWithTrailingSlash = 'http://www.squeaksource.com/Seaside29/'! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:47'!
testUrl
	gofer url: 'http://source.lukas-renggli.ch/pier'.
	self assert: (gofer repository isKindOf: MCHttpRepository).
	self assert: (gofer repository locationWithTrailingSlash = 'http://source.lukas-renggli.ch/pier/')! !

!GoferTest methodsFor: 'testing-options' stamp: 'lr 7/10/2009 16:55'!
testWiresong
	gofer wiresong: 'ob'.
	self assert: gofer repository locationWithTrailingSlash = 'http://source.wiresong.ca/ob/'! !
